
functor GlueFn(structure Bot : ROBOT) =
struct

    open World

    val showflood = Params.flag false (SOME ("-showflood", 
					     "Show the floodfill data (SLOW!)")) "showflood"

    val debug = Params.flag false (SOME ("-debug", 
					 "Turn on debugging messages.")) "debug"

    val _ =
    case Params.docommandline () of
	[host, port] =>
	    (let
		val _ = if !debug then print ("[GLUE] Contacting " ^ host ^ "\n") else ()
		val svr = Comm.contact (host, (valOf (Int.fromString port)))
		val _ = if !debug then print "[GLUE] Init...\n" else ()
		val {board as BOARD{robots,...}, me} = Comm.init svr
		val _ = if !debug then print "[GLUE] Init bot...\n" else ()
		val mind = Bot.init board (valOf (World.Map.find (robots, me)))

		val dmap = FloodFill.makemap board
		val _ = FloodFill.updatemap_id dmap board me

		(* one thing to do here is to soup up the data we send to the Bot.
		   For instance, we can use floodfill to calculate the distance to
		   each point and the direction to take to get there. *)

		fun loop (board as BOARD{robots,...}) mind =
		    let
			val _ = if !debug then print "[GLUE] movestart ...\n" else ()
			val (board as BOARD{robots,...}) = Util.time "movestart" (Comm.movestart svr me dmap) board
			val _ = if !debug then print "[GLUE] floodfill ...\n" else ()
			val _ = Util.time "floodfill" (FloodFill.updatemap_id dmap board) me
 			val _ = if !showflood then (print "SHOWING FLOOD:\n";
						    print (FloodFill.mapstring dmap board)) else ()
			val _ = if !debug then print "[GLUE] bot.move ...\n" else ()
			val (mind, bid, action) = 
			    Util.time "bot.move" (Bot.move board dmap mind) (valOf (World.Map.find (robots, me)))
		    in
			if !debug then print "[GLUE] comm.move ... \n" else ();
			loop (Comm.move svr me (bid, action) board) mind
		    end
	    in
		loop board mind
	    end handle e as (LSocket.Socket s) => (print ("Socket Error: " ^ s ^ "\n");
						   raise e))
		
      | _ => (print "usage:\n./robot host port\n\n";
	      print (Params.usage ()))

end
