structure World =
  struct
    datatype 'a knowledge = Known of 'a | Unknown

    datatype packageloc = Ground of int * int | Robot of int

    datatype package = PACKAGE of {dest : (int * int) knowledge,
                                   weight : int knowledge,
				   loc : packageloc knowledge,
                                   id : int}

    datatype robot = ROBOT of {loc : int * int,
                               money : int,
                               packages : int list,
                               scorelb : int}

    datatype space = Wall | Water | Dot of bool

    datatype spot = SPOT of {space : space,
                             robots : int list}

    structure Map = SplayMapFn (struct
				    type ord_key = int
				    val compare = Int.compare
				end)

    datatype board = BOARD of {layout : spot Array.array * int * int,
                               cap : int * int,
                               robots : robot Map.map,
                               packages : package Map.map}

    datatype direction = N | S | E | W

    datatype action =
      Move of direction
    | Pick of package list
    | Drop of package list

    fun getspot (SPOT {space, ...}) = space

    fun ht (b : spot Array.array, w : int, h : int) = h
    fun wi (b : spot Array.array, w : int, h : int) = w
    fun brd (b : spot Array.array, w : int, h : int) = b
    fun sub (b : spot Array.array, w : int, h : int) (r, c) =
      Array.sub (b, r * w + c)
    fun update (b : spot Array.array, w : int, h : int) (r, c) v =
      Array.update (b, r * w + c, v)
  end
